function [valuepde,valuedis,valuemc]=bondvasicek
% In this exercise we compute the bond price at time 0 in a Vasicek
% shortrate model
tic
%% parameter input
% horizon
T=10;
% sample size
Nsimu=10^5;
Nplot=Nsimu;
% grid points
M=10^3;
% volatility
sigma=0.04;
beta=-0.86;
b=0.08;
% mean reversion speed = lambda=-beta
lambda=-beta;
% mean reversion level = nu=-b/beta
nu=-b/beta;
% initial value = r_0
r0=0.08;
% time step
dt= T/M;

%% Analytical approach
AT= sigma^2*(4*exp(beta*T)-exp(2*beta*T)-2*beta*T-3)/(4*beta^3)+b*(exp(beta*T)-1-beta*T)/(beta^2);
BT= (exp(beta*T)-1)/beta;
valuepde=exp(-AT-BT*r0);

%% Distribution approach
mu0= r0/beta*(exp(beta*T)-1)+b/(beta^2)*(exp(beta*T)-1-beta*T);
sigma0sq= sigma^2/(2*beta^3)*(-4*exp(beta*T)+exp(2*beta*T)+2*beta*T+3);

% generate Nsimu*standard normal random variables
stdnormal = randn(1,Nsimu);
valuedis= mean(exp(-mu0+sqrt(sigma0sq)*stdnormal));


%% Monte Carlo approach
% BM
BM = [zeros(1,Nplot);sqrt(T/M)*cumsum(randn(M,Nplot))];
OU = [r0*ones(1,Nplot);zeros(M,Nplot)];
% Euler Maruyama Scheme
for i =1:M
    OU(i+1,:)=OU(i,:)+lambda*(nu-OU(i,:))*dt+ sigma.*(BM(i+1,:)-BM(i,:));
end

integral = zeros(1,Nsimu);
intgrid= 0:dt:T;
for j= 1:Nsimu
    % use trapezoidal rule
    integral(j)= trapz(intgrid,OU(:,j));
end
% take the expectation
valuemc= mean(exp(-integral));
toc